/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import java.util.Map;

import org.apache.tools.ant.BuildException;

/**
 * <p>
 * This class represents nested element of the "credentials" element.
 * Instances of CredentialProperty are expected to be created by CredentialsElement instance,
 * this class is present for ant engine to save element data in build file.
 * </p>
 *
 * <p>
 * This nested element represents single property from the property map.
 * The name of nested element represents the property key, the corresponding value is set by the corresponding
 * "value" attribute.
 * </p>
 *
 * <p>
 * Thread Safety : This class is mutable and so not thread-safe.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 *
 */
public class CredentialProperty {
    /**
     * <p>
     * Represents the name of property, represented by this element.
     * Is initialized in the constructor.
     * </p>
     *
     */
    private final String name;

    /**
     * <p>
     * Represents the property map, this property belongs to.
     * Is initialized in the constructor.
     * </p>
     *
     */
    private final Map propertyMap;

    /**
     * <p>
     * Creates an instance of CredentialProperty class.
     * </p>
     *
     * <p>
     * The name of property and the property map, the property should belong to, are specified.
     * </p>
     *
     * @param name the name of the property
     * @param propertyMap the property map, the property  should belong to
     *
     * @throws IllegalArgumentException if name or propertyMap is null
     * @throws IllegalArgumentException if name is empty string
     */
    public CredentialProperty(String name, Map propertyMap) {
        if (name == null) {
            throw new IllegalArgumentException("name is null.");
        }
        if (name.trim().length() == 0) {
            throw new IllegalArgumentException("name is empty.");
        }
        if (propertyMap == null) {
            throw new IllegalArgumentException("propertyMap is null.");
        }

        this.name = name;
        this.propertyMap = propertyMap;
    }

    /**
     * <p>
     * Sets the value of "value" attribute.
     * </p>
     *
     * <p>
     * "value" attribute represents the value of the property.
     * This method adds property to the property map.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * The value argument can be empty.
     * </p>
     *
     * @param value the value of the property to be set
     *
     * @throws BuildException if credentials element has two nest elements with equal tag name.
     */
    public void setValue(String value) {
        // if name exists in propertyMap, it tells that two elements with equal tag name exist
        if (propertyMap.containsKey(this.name)) {
            throw new BuildException("The " + this.name + " element under credentials element occurs more than once.");
        } else {
            propertyMap.put(this.name, value);
        }
    }
}
