/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import java.util.HashMap;
import java.util.Map;
import org.apache.tools.ant.DynamicElement;

/**
 * <p>
 * This class represents a "credentials" XML element to be used in the Ant build file.
 * </p>
 *
 * <p>
 * The credentials can include user name, password and other information needed to authenticate to the server.
 * A particular Server implementation should require the specific credentials.
 * </p>
 *
 * <p>
 * This element represents some kind of a property map, each property is described by one nested element.
 * The name of nested element represents the property key, the corresponding value is set by the corresponding
 * "value" attribute.
 * <p>
 *
 * <p>
 * Example of the credentials element:
 * </p>
 * <p>
 * &lt;credentials&gt;
 *      &lt;user_name value=&quot;topcoder&quot; /&gt;
 *      &lt;password value=&quot;some_password&quot; /&gt;
 *      &lt;some_other_property=&quot;some_value&quot;/&gt;
 * &lt;/credentials&gt;
 * </p>
 *
 * <p>
 * Thread Safety : This class is mutable and so not thread-safe.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class CredentialsElement implements DynamicElement {
    /**
     * <p>
     * Stores the property map represented by the "credentials" element.
     * </p>
     *
     * <p>
     * Keys should be String instances, values should also be String instances.
     * They are set via call to the CredentialProperty.setValue().
     * </p>
     *
     */
    private Map propertyMap = new HashMap();

    /**
     * <p>
     * Creates an instance of CredentialsElement class.
     * </p>
     *
     */
    public CredentialsElement() {
    }

    /**
     * <p>
     * Creates a nested element with a specified name.
     * Nested elements are of type CredentialProperty.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param name the name of nested element to create
     * @return the created CredentialProperty instance
     *
     * @throws IllegalArgumentException if name is null or empty
     */
    public Object createDynamicElement(String name) {
        if (name == null) {
            throw new IllegalArgumentException("name is null.");
        }
        if (name.trim().length() == 0) {
            throw new IllegalArgumentException("name is empty.");
        }

        return new CredentialProperty(name, this.propertyMap);
    }

    /**
     * <p>
     * Returns the property map represented by the "credentials" element.
     * </p>
     *
     * @return  the property map represented by the "credentials" element
     */
    public Map getPropertyMap() {
        return this.propertyMap;
    }
}
