/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import java.io.File;
import org.apache.cactus.integration.ant.CactifyWarTask;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.ZipFileSet;

/**
 * <p>
 * This class represents an Ant task that injects elements necessary to run the tests into an existing
 * WAR file.
 * </p>
 *
 * <p>
 * It can be used to prepare WARs for DefaultWebApplicationServer class.
 * </p>
 *
 * <p>
 * Note, for now it's just a wrapper for CactifyWarTask, but future implementations may be different.
 * This class provides an interface that is required for the future implementations.
 * </p>
 *
 * <p>
 * Thread Safety : This class is mutable and so not thread-safe.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class PrepareTestWarTask extends Task {

    /**
     * <p>
     * Stores wrapped CactifyWarTask instance.
     * Is initialized when declared and then never changed.
     * </p>
     *
     */
    private final CactifyWarTask cactifyWarTask = new CactifyWarTask();

    /**
     * <p>
     * Default constructor.
     * Creates a new PrepareTestWarTask instance.
     * </p>
     *
     */
    public PrepareTestWarTask() {
    }

    /**
     * <p>
     * Sets the value of "srcfile" attribute.
     * </p>
     *
     * <p>
     * "srcfile" attribute specifies the source archive to inject the elements required to run tests into.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param srcFile the source archive to inject the elements required to run tests into
     */
    public void setSrcFile(File srcFile) {
        cactifyWarTask.setSrcFile(srcFile);
    }

    /**
     * <p>
     * Sets the value of "destfile" attribute.
     * </p>
     *
     * <p>
     * "destfile" attribute specifies the destination file, archive with the injected elements required
     * to run tests will be written to.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param destFile the destination file, archive with the injected elements required to run tests
     * will be written to
     */
    public void setDestFile(File destFile) {
        cactifyWarTask.setDestFile(destFile);
    }

    /**
     * <p>
     * Adds a set of files to the destination file, under WEB-INF/lib/.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param fs a set of files to add under WEB-INF/lib/
     */
    public void addLib(ZipFileSet fs) {
        cactifyWarTask.addLib(fs);
    }

    /**
     * <p>
     * Adds a set of files to the destination file, under WEB-INF/classes/.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param fs a set of files to add under WEB-INF/classes/
     */
    public void addClasses(ZipFileSet fs) {
        cactifyWarTask.addClasses(fs);
    }

    /**
     * <p>
     * Adds a set of files to the destination file.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param set a set of files to add to the destination file.
     */
    public void addFileset(FileSet set) {
        cactifyWarTask.addFileset(set);
    }

    /**
     * <p>
     * Runs the task.
     * </p>
     *
     * <p>
     * Simply delegate it to wrapped class instance.
     * </p>
     *
     * @throws org.apache.tools.ant.BuildException if any error happens
     */
    public void execute() {
        // set up the connection between ant and cactifyWarTask, which is necessary for executing an ant task.
        cactifyWarTask.setProject(getProject());
        cactifyWarTask.setLocation(getLocation());
        cactifyWarTask.setTaskName(getTaskName());
        cactifyWarTask.setOwningTarget(getOwningTarget());
        cactifyWarTask.setTaskType(getTaskType());

        cactifyWarTask.execute();
    }
}
