/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import org.apache.cactus.FilterTestCase;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.http.HttpServletResponse;

/**
 * <p>
 * This class represents JUnit test case to unit test code that needs an access to valid Filter implicit
 * objects (such as the FilterConfig and FilterChain objects).
 * </p>
 *
 * <p>
 * This implementation of the class is derived from Cactus FilterTestCase, and just adds more high level access
 * to Filter objects.
 * Filter objects should be accessed by the derived classes using the getter methods provided by this class.
 * Derived test cases should not use directly methods and variables defined by the Cactus framework.
 * </p>
 *
 * <p>
 * For each test case, you can define a corresponding beginXXX() method(optional).
 * It will be run on the client side and can be used to initialize HTTP related parameters
 * (HTTP parameters, cookies, HTTP headers, URL to simulate, ...).
 * You will be able to retrieve these values in your testXXX() by calling
 * the corresponding API of HttpServletRequest (like getParameter(), getCookies(), getHeader(), ...).
 *
 * The signature of the begin method is:
 * <pre>
 * public void beginXXX(WebRequest theRequest)
 * {
 *   [...]
 * }
 * </pre>
 * </p>
 *
 * <p>
 * For each test case, you can define a corresponding endXXX() method(optional).
 * This method is called on thee client side and can be used to verify the returned HTTP related parameters
 * from your test case (like the returned content of the HTTP response, any returned cookies, returned HTTP headers,
 * ...).
 *
 * The signature of the end method is:
 * <pre>
 * public void endXXX(WebResponse theResponse)
 * {
 *   [...]
 * }
 * </pre>
 * </p>
 *
 * <p>
 * Note, the setUp(), tearDown() and testXXX() methods are run on the server side.
 * </p>
 *
 * <p>
 * Thread Safety : This class is mutable and so not thread-safe.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class TCFilterTestCase extends FilterTestCase {
    /**
     * <p>
     * Default constructor defined in order to allow creating Test Case without needing to
     * define constructor(new feature in JUnit 3.8.1).
     * </p>
     *
     */
    public TCFilterTestCase() {
    }

    /**
     * <p>
     * Constructs a test case with the given name.
     * <p>
     *
     * @param name the name of the test case
     */
    public TCFilterTestCase(String name) {
        super(name);
    }

    /**
     * <p>
     * Returns a valid FilterConfig object that you can access from the
     * testXXX(), setUp() and tearDown() methods.
     * If you try to access it from either the beginXXX() or endXXX() methods
     * it will have the null value.
     * </p>
     *
     * @return a valid FilterConfig object
     */
    public FilterConfig getConfig() {
        return this.config;
    }

    /**
     * <p>
     * Returns a valid FilterChain object that you can access from the
     * testXXX(), setUp() and tearDown() methods.
     * If you try to access it from either the beginXXX() or endXXX() methods
     * it will have the null value.
     * </p>
     *
     * @return a valid FilterChain object
     */
    public FilterChain getFilterChain() {
        return this.filterChain;
    }

    /**
     * <p>
     * Returns a valid HttpServletRequest object that you can access from the
     * testXXX(), setUp() and tearDown() methods.
     * If you try to access it from either the beginXXX() or endXXX() methods
     * it will have the null value.
     * </p>
     *
     * @return a valid HttpServletRequest object
     */
    public HttpServletRequest getRequest() {
        return this.request;
    }

    /**
     * <p>
     * Returns a valid HttpServletResponse object that you can access from the
     * testXXX(), setUp() and tearDown() methods.
     * If you try to access it from either the beginXXX() or endXXX() methods
     * it will have the null value.
     * </p>
     *
     * @return a valid HttpServletResponse object
     */
    public HttpServletResponse getResponse() {
        return this.response;
    }
}
