/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Enumeration;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import org.apache.tools.ant.BuildException;

/**
 * <p>
 * This class extends DataServer and is used for HSQL Database.
 * </p>
 *
 * <p>
 * Note, it's only for demo purpose. In this implementation, The SERVER mode is used to run HSQLDB server,
 * and the default test database.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class HSQLDataServer extends DataServer {
    /**
     * <p>
     * HSQLDB Server instance for starting or shutting HSQLDB Server.
     * It is initialed in startUp() method and will be set null in shutDown() method.
     * </p>
     */
    private org.hsqldb.Server server;

    /**
     * <p>
     * Creates a new HSQLDataServer instance.
     * Only hsqldb type is allowed and other types are banned.
     * </p>
     *
     * @param type it should be "hsqldb" to identify using hsql database.
     *
     * @throws BuildException when the type is not "hsqldb".
     */
    public HSQLDataServer(String type) {
        super(type);

        // this class is only used for HSQLDB database.
        if (!"hsqldb".equalsIgnoreCase(type)) {
            throw new BuildException("The type should be only hsqldb.");
        }
    }

    /**
     * <p>
     * Starts a HSQLDB Server.
     * </p>
     */
    public void startUp() {
        server = new org.hsqldb.Server();
        server.start();
    }

    /**
     * <p>
     * Stops a HSQLDB Server.
     * </p>
     *
     * @throws org.apache.tools.ant.BuildException when SQLException occurs
     */
    public void shutDown() {
        // get the database config value.
        // Note, this implementation uses user_name and password properties.
        Map map = this.getCredentials().getPropertyMap();
        String user = (String) map.get(Server.USER_NAME);
        String pwd = (String) map.get(Server.PASSWORD);

        // shut down the database
        Connection connection = null;
        try {
            connection = TestHelper.getConnection(user, pwd);
            executeUpdate(connection, "SHUTDOWN");
        } catch (SQLException e) {
            throw new BuildException("SQLException occurs", e);
        } finally {
            TestHelper.closeConnection(connection);
        }

        // stop server
        if (server != null) {
            server.stop();
            server = null;
        }
    }

    /**
     * <p>
     * Initializes the test database using the sql files specified in the jar file.
     * Note, names of the sql files using in this method must be started with <b>create</b>.
     * </p>
     *
     * @throws BuildException when ZipException or IOException or SQLException occurs.
     */
    public void initData() {
        operateData("create");
    }

    /**
     * <p>
     * Cleaning the test database using the sql files specified in the jar file.
     * Note, names of the sql files using in this method must be started with <b>drop</b>.
     * </p>
     *
     * @throws BuildException when ZipException or IOException or SQLException occurs.
     */
    public void cleanUpData() {
        operateData("drop");
    }

    /**
     * <p>
     * Executes the sql files contained in the jar file.
     * The given argument prefix is used to filter which sql files will be used.
     * </p>
     *
     * @param prefix is used to filter which sql files will be used.
     *
     * @throws org.apache.tools.ant.BuildException when ZipException or IOException or SQLException occurs.
     */
    private void operateData(String prefix) {
        // get the database config value.
        // Note, this implementation uses user_name and password properties.
        Map map = this.getCredentials().getPropertyMap();
        String user = (String) map.get(Server.USER_NAME);
        String pwd = (String) map.get(Server.PASSWORD);

        Connection connection = null;
        try {
            connection = TestHelper.getConnection(user, pwd);

            ZipFile zipFile = new ZipFile(this.getJarFile());
            Enumeration e = zipFile.entries();

            // iterates all the entries in the jar file.
            while (e.hasMoreElements()) {
                ZipEntry ze = (ZipEntry) e.nextElement();

                // only the file name started with drop is used for cleaning operation.
                if (ze.getName().startsWith(prefix)) {
                    String commands = readFileFromJar(zipFile, ze);

                    // a sql file can contain several sql expressions
                    // they are separated by a comma.
                    String[] sqlArray = commands.split(";");
                    for (int i = 0; i < sqlArray.length; i++) {
                        if (sqlArray[i] != null && sqlArray[i].trim().length() != 0) {
                            executeUpdate(connection, sqlArray[i]);
                        }
                    }
                }
            }

        } catch (ZipException e) {
            throw new BuildException("ZipException occurs", e);
        } catch (IOException e) {
            throw new BuildException("IOException occurs", e);
        } catch (SQLException e) {
            throw new BuildException("SQLException occurs", e);
        } finally {
            TestHelper.closeConnection(connection);
        }
    }

    /**
     * <p>
     * Reads a text file contained in a zip file.
     * </p>
     *
     * @param zipFile ZipFile instance
     * @param entry the entry to read a file from zipFile.
     * @return the text content of the specified file contained in zipFile
     *
     * @throws org.apache.tools.ant.BuildException when ZipException or IOException occurs.
     */
    private String readFileFromJar(ZipFile zipFile, ZipEntry entry) {
        StringBuffer content = new StringBuffer(1024);
        try {
            // get the corresponding InputStream instance and buffer it
            InputStream is = zipFile.getInputStream(entry);
            BufferedReader br = new BufferedReader(new InputStreamReader(is));

            // read the content line by line
            String line = null;
            while ((line = br.readLine()) != null) {
                content.append(line);
            }

            return content.toString();
        } catch (ZipException e) {
            throw new BuildException("ZipException occurs", e);
        } catch (IOException e) {
            throw new BuildException("IOException occurs", e);
        }
    }

    /**
     * <p>
     * Executes a sql expression(update operation instead of query operation).
     * </p>
     *
     * @param connection database operation connection
     * @param sql the sql expression to execute
     * @throws SQLException when any exception occurs during database operation
     */
    private void executeUpdate(Connection connection, String sql) throws SQLException {
        Statement statement = connection.createStatement();

        statement.executeUpdate(sql);

        // release database resource
        if (statement != null) {
            statement.close();
        }
    }
}
