/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.ProjectHelper;
import org.apache.tools.ant.Target;

/**
 * <p>
 * A helper class to perform those common operations which is helpful for the test.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 *
 */
public class TestHelper {
    /**
     * <p>
     * A sample build.xml used for testing.
     * </p>
     */
    public static final String SAMPLE_BUILD_XML = "test_files/sample_build.xml";

    /**
     * <p>
     * The tomcat 5x web server install directory.
     * </p>
     */
    public static final String TOMCAT5X_HOME = "C:\\Program Files\\Apache Software Foundation\\Tomcat 5.0";

    /**
     * <p>
     * This private constructor prevents to create a new instance.
     * </p>
     */
    private TestHelper() {
    }

    /**
     * <p>
     * Returns a Project instance representing the build xml file.
     * </p>
     *
     * @param buildFile the build xml file to parse into project instance.
     * @return a Project instance representing the build xml file.
     *
     * @throws Exception to JUnit.
     */
    public static Project getProject(String buildFile) throws Exception {
        // create and init a new Project instance.
        Project project =  new Project();
        project.init();

        // config the project using the given build xml file.
        ProjectHelper helper = ProjectHelper.getProjectHelper();
        helper.parse(project, new File(buildFile));
        return project;
    }

    /**
     * <p>
     * Returns a new AntTaskInfo instance for testing.
     * </p>
     *
     * @return a new AntTaskInfo instance for testing.
     *
     * @throws Exception to JUnit.
     */
    public static AntTaskInfo getAntTaskInfo() throws Exception {
        Project project = TestHelper.getProject(TestHelper.SAMPLE_BUILD_XML);

        Location location = new Location(TestHelper.SAMPLE_BUILD_XML);
        Target target = (Target) project.getTargets().get("compile");

        AntTaskInfo antTaskInfo = new AntTaskInfo(project, "javac", location, target);

        return antTaskInfo;
    }

    /**
     * <p>
     * Returns a Connection instance for HSQLDB Database.
     * </p>
     *
     * @param user the user name to connect HSQLDB Database.
     * @param pwd the user password to connect HSQLDB Database.
     * @return a Connection instance for HSQLDB Database.
     *
     * @throws SQLException to JUnit
     */
    public static Connection getConnection(String user, String pwd) throws SQLException {
        try {
            Class.forName("org.hsqldb.jdbcDriver");
        } catch (ClassNotFoundException e) {
            throw new BuildException("ClassNotFoundException occurs.", e);
        }
        return DriverManager.getConnection("jdbc:hsqldb:hsql://localhost", user, pwd);
    }

    /**
     * <p>
     * Closes a Connection instance silently.
     * </p>
     *
     * @param connection used for database operation.
     */
    public static void closeConnection(Connection connection) {
        try {
            if (connection != null && !connection.isClosed()) {
                connection.close();
            }
        } catch (SQLException e) {
            // ignore
        }
    }

    /**
     * <p>
     * Deletes all the temporary files while generated in test cases.
     * </p>
     *
     */
    public static void clearTempFiles() {
        // remove the temp files in test_files/temp
        File path = new File("test_files/temp");
        File[] files = path.listFiles();
        for (int i = 0; i < files.length; i++) {
            if (files[i].isFile()) {
                files[i].delete();
            }
        }

        // remove the hsql database files
        new File("test.script").delete();
        new File("test.data").delete();
        new File("test.backup").delete();
        new File("test.properties").delete();
        new File("test.log").delete();
        new File("test.lck").delete();
    }
}
