/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import java.io.File;
import java.net.ConnectException;
import java.net.HttpURLConnection;
import java.net.URL;

import org.apache.tools.ant.BuildException;

import com.topcoder.testframework.TestHelper;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

/**
 * <p>
 * Unit test cases for DefaultWebApplicationServer.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */

public class DefaultWebApplicationServerTests extends TestCase {
    /**
     * <p>
     * Type constant used for constructing DefaultWebApplicationServer instance.
     * </p>
     */
    private static final String TYPE = "tomcat5x";

    /**
     * <p>
     * DefaultWebApplicationServer instance for testing.
     * </p>
     */
    private DefaultWebApplicationServer server;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     */
    protected void setUp() {
        server = new DefaultWebApplicationServer(TYPE);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     */
    protected void tearDown() {
        server = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(DefaultWebApplicationServerTests.class);
    }

    /**
     * <p>
     * Tests ctor DefaultWebApplicationServer#DefaultWebApplicationServer(String) for accuracy.
     * Expects a new DefaultWebApplicationServer instance created successfully.
     * </p>
     */
    public void testCtor() {
        assertNotNull("Failed to create new DefaultWebApplicationServer instance.", server);
    }

    /**
     * <p>
     * Tests ctor DefaultWebApplicationServer#DefaultWebApplicationServer(String) for failure.
     * It tests the case that when type is null and expects IllegalArgumentException.
     * </p>
     */
    public void testCtor_NullType() {
        try {
            new DefaultWebApplicationServer(null);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor DefaultWebApplicationServer#DefaultWebApplicationServer(String) for failure.
     * It tests the case that when type is empty and expects IllegalArgumentException.
     * </p>
     */
    public void testCtor_EmptyType() {
        try {
            new DefaultWebApplicationServer(" ");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests ctor DefaultWebApplicationServer#DefaultWebApplicationServer(String) for failure.
     * It tests the case that when type is unknown and expects BuildException.
     * </p>
     */
    public void testCtor_UnknownType() {
        try {
            new DefaultWebApplicationServer("topcoderServer");
            fail("BuildException expected.");
        } catch (BuildException be) {
            //good
        }
    }

    /**
     * <p>
     * Tests DefaultWebApplicationServer#startUp() and DefaultWebApplicationServer#shutDown()
     * for accuracy.
     * </p>
     *
     * <p>
     * In this test case, tomcat is started and expects it can be connected.
     * Then the tomcat is stopped and expects ConnectException when trying to connect tomcat server.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testStartUp() throws Exception {
        URL url = new URL("http://localhost:8080/sample/index.jsp");

        try {
            server.setDir(new File(TestHelper.TOMCAT5X_HOME));
            server.setWarFile(new File("test_files/sample.war"));
            server.setAntTaskInfo(TestHelper.getAntTaskInfo());

            server.setPort(8080);
            server.setTimeout(30000);

            server.startUp();

            assertEquals("Expects the tomcat web server is accessible.",
                    HttpURLConnection.HTTP_OK, getResponseCode(url));
        } finally {

            server.shutDown();

            try {
                getResponseCode(url);
                fail("ConnectException expected.");
            } catch (ConnectException ce) {
                // good
            }
        }
    }

    /**
     * <p>
     * Tests DefaultWebApplicationServer#startUp() for fail.
     * It tests the case when dir attribute is not specified and expects BuildException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testStartUp_NoDir() throws Exception {
        try {
            server.setWarFile(new File("test_files/sample.war"));
            server.setAntTaskInfo(TestHelper.getAntTaskInfo());

            server.setPort(8080);
            server.setTimeout(30000);

            server.startUp();

            fail("BuildException expected.");
        } catch (BuildException be) {
            // good
        }
    }

    /**
     * <p>
     * Tests DefaultWebApplicationServer#startUp() for fail.
     * It tests the case when none of warFile and testURL attributes is specified and expects BuildException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testStartUp_NoWarFileNoTestURL() throws Exception {
        try {
            server.setDir(new File(TestHelper.TOMCAT5X_HOME));
            server.setAntTaskInfo(TestHelper.getAntTaskInfo());

            server.setPort(8080);
            server.setTimeout(30000);

            server.startUp();

            fail("BuildException expected.");
        } catch (BuildException be) {
            // good
        }
    }

    /**
     * <p>
     * Tests DefaultWebApplicationServer#startUp() for fail.
     * It tests the case when antTaskInfo is not specified and expects BuildException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testStartUp_NoAntTaskInfo() throws Exception {
        try {
            server.setDir(new File(TestHelper.TOMCAT5X_HOME));
            server.setWarFile(new File("test_files/sample.war"));

            server.setPort(8080);
            server.setTimeout(30000);

            server.startUp();

            fail("BuildException expected.");
        } catch (BuildException be) {
            // good
        }
    }

    /**
     * <p>
     * Tests DefaultWebApplicationServer#initData() for accuracy.
     * Trivial test, in current implementation this method does nothing.
     * </p>
     */
    public void testInitData() {
        server.initData();
    }

    /**
     * <p>
     * Tests DefaultWebApplicationServer#cleanUpData() for accuracy.
     * Trivial test, in current implementation this method does nothing.
     * </p>
     */
    public void testCleanUpData() {
        server.cleanUpData();
    }

    /**
     * <p>
     * Tests DefaultWebApplicationServer#setTimeout(long) for accuracy.
     * Expects timeout is correctly set.
     * </p>
     */
    public void testSetTimeout() {
        server.setTimeout(500000);

        assertEquals("Failed to get correct timeout value.", 500000, server.getTimeout());
    }

    /**
     * <p>
     * Tests DefaultWebApplicationServer#setTimeout(long) for failure.
     * It tests the case when timeout is negative and expects IllegalArgumentException.
     * </p>
     */
    public void testSetTimeout_Negative() {
        try {
            server.setTimeout(-500000);
        } catch (IllegalArgumentException iae) {
            //good
        }
    }

    /**
     * <p>
     * Tests DefaultWebApplicationServer#getTimeout() for accuracy.
     * Expects the default timeout value is 180000.
     * </p>
     */
    public void testGetTimeout() {
        assertEquals("Failed to get correct default timeout value.", 180000, server.getTimeout());
    }

    /**
     * <p>
     * Tests DefaultWebApplicationServer#setTestURL(URL) for accuracy.
     * Expects the test url is correctly set.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testSetTestURL() throws Exception {
        URL url = new URL("http://localhost:8080/test");
        server.setTestURL(url);

        assertEquals("Failed to get correct protocol.", "http", server.getTestURL().getProtocol());
        assertEquals("Failed to get correct port.", 8080, server.getTestURL().getPort());
        assertEquals("Failed to get correct path.", "/test", server.getTestURL().getPath());
    }

    /**
     * <p>
     * Tests DefaultWebApplicationServer#getTestURL() for accuracy.
     * Expects the test url is initially null.
     * </p>
     */
    public void testGetTestURL() {
        assertNull("Expects null testURL.", server.getTestURL());
    }

    /**
     * <p>
     * Returns the response code using HTTP Connection.
     * Note, when the http connection is unavailable, exception is expected.
     * </p>
     *
     * @param url the url address to connect
     * @return the response code
     *
     * @throws Exception to JUnit.
     */
    private int getResponseCode(URL url) throws Exception {
        HttpURLConnection http = null;
        try {
            // Create the connection
            http = (HttpURLConnection) url.openConnection();
            return http.getResponseCode();
        } finally {
            // disconnect the connection
            if (http != null) {
                http.disconnect();
            }
        }
    }

}